def test_imports():
    from symconf.runner import Runner
    from symconf.reader import DictReader
    from symconf.config import ConfigManager
    from symconf.matching import Matcher, FilePart
    from symconf.template import Template, FileTemplate, TOMLTemplate

    from symconf import config
    from symconf import reader
    from symconf import util



import argparse
from importlib.metadata import version

from symconf import util, __version__
from symconf.config import ConfigManager


def add_install_subparser(subparsers):
    def install_apps(args):
        cm = ConfigManager(args.config_dir)
        cm.install_apps(apps=args.apps)

    parser = subparsers.add_parser(
        'install',
        description='Run install scripts for registered applications.'
    )
    parser.add_argument(
        '-a', '--apps',
        required = False,
        default  = "*",
        type     = lambda s: s.split(',') if s != '*' else s,
        help     = 'Application target for theme. App must be present in the registry. ' \
                 + 'Use "*" to apply to all registered apps'
    )
    parser.set_defaults(func=install_apps)

def add_update_subparser(subparsers):
    def update_apps(args):
        cm = ConfigManager(args.config_dir)
        cm.update_apps(apps=args.apps)

    parser = subparsers.add_parser(
        'update',
        description='Run update scripts for registered applications.'
    )
    parser.add_argument(
        '-a', '--apps',
        required = False,
        default  = '*',
        type     = lambda s: s.split(',') if s != '*' else s,
        help     = 'Application target for theme. App must be present in the registry. ' \
                 + 'Use "*" to apply to all registered apps'
    )
    parser.set_defaults(func=update_apps)

def add_config_subparser(subparsers):
    def configure_apps(args):
        cm = ConfigManager(args.config_dir)
        cm.configure_apps(
            apps=args.apps,
            scheme=args.mode,
            style=args.style,
        )

    parser = subparsers.add_parser(
        'config',
        description='Set config files for registered applications.'
    )
    parser.add_argument(
        '-s', '--style',
        required = False,
        default  = 'any',
        help     = 'Style indicator (often a color palette) capturing thematic details in '
                   'a config file'
    )
    parser.add_argument(
        '-m', '--mode',
        required = False,
        default  = "any",
        help     = 'Preferred lightness mode/scheme, either "light," "dark," "any," or "none."'
    )
    parser.add_argument(
        '-a', '--apps',
        required = False,
        default  = "*",
        type     = lambda s: s.split(',') if s != '*' else s,
        help     = 'Application target for theme. App must be present in the registry. ' \
                 + 'Use "*" to apply to all registered apps'
    )
    parser.add_argument(
        '-T', '--template-vars',
        required = False,
        nargs='+',
        action=util.KVPair,
        help='Groups to use when populating templates, in the form group=value'
    )
    parser.set_defaults(func=configure_apps)

def add_generate_subparser(subparsers):
    def generate_apps(args):
        cm = ConfigManager(args.config_dir)
        cm.generate_app_templates(
            gen_dir=args.gen_dir,
            apps=args.apps,
        )

    parser = subparsers.add_parser(
        'generate',
        description='Generate all template config files for specified apps'
    )
    parser.add_argument(
        '-g', '--gen-dir',
        required = True,
        type     = util.absolute_path,
        help     = 'Path to write generated template files'
    )
    parser.add_argument(
        '-a', '--apps',
        required = False,
        default  = "*",
        type     = lambda s: s.split(',') if s != '*' else s,
        help     = 'Application target for theme. App must be present in the registry. ' \
                 + 'Use "*" to apply to all registered apps'
    )
    parser.set_defaults(func=generate_apps)


# central argparse entry point
parser = argparse.ArgumentParser(
    'symconf',
    description='Manage application configuration with symlinks.'
)
parser.add_argument(
    '-c', '--config-dir',
    default = util.xdg_config_path(),
    type    = util.absolute_path,
    help    = 'Path to config directory'
)
parser.add_argument(
    '-v', '--version',
    action='version',
    version=__version__,
    help    = 'Print symconf version'
)

# add subparsers
subparsers = parser.add_subparsers(title='subcommand actions')
add_config_subparser(subparsers)
add_generate_subparser(subparsers)
add_install_subparser(subparsers)
add_update_subparser(subparsers)


def main():
    args = parser.parse_args()

    if 'func' in args:
        args.func(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



import stat
import subprocess
from pathlib import Path

from colorama import Fore, Back, Style

from symconf.util import printc, color_text


class Runner:
    def run_script(
        self,
        script: str | Path,
    ):
        script_path = Path(script)

        if script_path.stat().st_mode & stat.S_IXUSR == 0:
            print(
                color_text("│", Fore.BLUE),
                color_text(
                    f'   > script "{script_path.name}" missing execute permissions, skipping',
                    Fore.RED + Style.DIM
                )
            )
            return

        print(
            color_text("│", Fore.BLUE),
            color_text(
                f' > running script "{script_path.name}"',
                Fore.BLUE
            )
        )

        output = subprocess.check_output(str(script_path), shell=True)

        if output:
            fmt_output = output.decode().strip().replace(
                '\n',
                f'\n{Fore.BLUE}{Style.NORMAL}│{Style.DIM}    '
            )
            print(
                color_text("│", Fore.BLUE),
                color_text(
                    f'   captured script output "{fmt_output}"',
                    Fore.BLUE + Style.DIM
                )
            )

        return output

    def run_many(
        self,
        script_list: list[str | Path],
    ):
        outputs = []
        for script in script_list:
            output = self.run_script(script)
            outputs.append(output)

        return outputs



import re
import tomllib
from pathlib import Path

from symconf import util
from symconf.reader import DictReader


class Template:
    def __init__(
        self,
        template_str  : str,
        pattern       : str = r'f{{(\S+?)}}',
    ):
        self.template_str = template_str
        self.pattern = pattern

    def fill(
        self,
        template_dict : dict,
    ) -> str:
        dr = DictReader.from_dict(template_dict)

        return re.sub(
            self.pattern,
            lambda m: str(dr.get(m.group(1))),
            self.template_str
        )

class FileTemplate(Template):
    def __init__(
        self,
        path    : Path,
        pattern : str = r'f{{(\S+)}}',
    ):
        super().__init__(
            path.open('r').read(),
            pattern=pattern
        )

class TOMLTemplate(FileTemplate):
    def __init__(
        self,
        toml_path : Path,
        pattern   : str = r'f{{(\S+)}}',
    ):
        super().__init__(
            toml_path,
            pattern=pattern
        )

    def fill(
        self,
        template_dict : dict,
    ) -> str:
        filled_template = super().fill(template_dict)
        toml_dict = tomllib.loads(filled_template)

        return toml_dict

    @staticmethod
    def stack_toml(
        path_list: list[Path]
    ) -> dict:
        stacked_dict = {}
        for toml_path in path_list:
            updated_map = tomllib.load(toml_path.open('rb'))
            stacked_dict = util.deep_update(stacked_dict, updated_map)

        return stacked_dict



from pathlib import Path

from symconf import ConfigManager


config_dir = Path(
    __file__, '..', 'test-config-dir/'
).resolve()
cm = ConfigManager(config_dir)


def test_matching_configs_exact():
    '''
    Test matching exact style and scheme. Given strict mode not set (allowing relaxation
    to "none"), the order of matching should be

    1. (none, none)    :: none-none.aaa
    2. (none, scheme)  :: none-light.aaa
    3. (style, none)   :: test-none.aaa
    4. (style, scheme) :: test-light.ccc

    Yielding "test-light.aaa", "test-light.ccc" (unique only on config pathname).
    '''
    any_light = cm.get_matching_configs(
        'test', 
        style='test',
        scheme='light',
    )
    print(any_light)

    assert len(any_light) == 2
    assert any_light['aaa'].pathname == 'test-none.aaa'
    assert any_light['ccc'].pathname == 'test-light.ccc'

def test_matching_configs_any_style():
    '''
    Test matching "any" style and exact scheme. Given strict mode not set (allowing
    relaxation to "none"), the order of matching should be

    1. (style, none)   :: none-none.aaa, test-none.aaa
    2. (none, none)    :: none-none.aaa
    3. (style, scheme) :: test-dark.bbb
    4. (none, scheme)  :: (nothing)

    Yielding "none-none.aaa" (should always overwrite "test-none.aaa" due to "any"'s
    preference for non-specific matches, i.e., "none"s), "test-none.ddd", "test-dark.bbb"
    (unique only on config pathname).
    '''
    any_dark = cm.get_matching_configs(
        'test', 
        style='any',
        scheme='dark',
    )

    assert len(any_dark) == 2
    assert any_dark['aaa'].pathname == 'none-none.aaa'
    assert any_dark['bbb'].pathname == 'test-dark.bbb'

def test_matching_configs_any_scheme():
    '''
    Test matching exact style and "any" scheme. Given strict mode not set (allowing
    relaxation to "none"), the order of matching should be

    1. (none, scheme)  :: none-light.aaa & none-none.aaa
    2. (none, none)    :: none-none.aaa
    3. (style, scheme) :: test-dark.bbb & test-light.ccc & test-none.aaa
    4. (style, none)   :: test-none.aaa

    Yielding "test-none.aaa", "test-light.ccc", "test-dark.bbb"
    '''
    test_any = cm.get_matching_configs(
        'test', 
        style='test',
        scheme='any',
    )

    assert len(test_any) == 3
    assert test_any['aaa'].pathname == 'test-none.aaa'
    assert test_any['bbb'].pathname == 'test-dark.bbb'
    assert test_any['ccc'].pathname == 'test-light.ccc'

def test_matching_scripts():
    '''
    Test matching exact style and scheme. Given strict mode not set (allowing relaxation
    to "none"), the order of matching should be

    1. (none, none)    :: none-none.sh
    2. (none, scheme)  :: none-light.sh
    3. (style, none)   :: test-none.sh
    4. (style, scheme) :: (nothing)

    Yielding (ordered by dec specificity) "test-none.sh" as primary match, then relaxation
    match "none-none.sh".
    '''
    test_any = cm.get_matching_scripts(
        'test', 
        style='test',
        scheme='any',
    )

    assert len(test_any) == 2
    assert list(map(lambda p:p.pathname, test_any)) == ['test-none.sh', 'none-none.sh']

    any_light = cm.get_matching_scripts(
        'test', 
        style='any',
        scheme='light',
    )

    assert len(any_light) == 2
    assert list(map(lambda p:p.pathname, any_light)) == ['none-light.sh', 'none-none.sh']

    any_dark = cm.get_matching_scripts(
        'test', 
        style='any',
        scheme='dark',
    )

    assert len(any_dark) == 2
    assert list(map(lambda p:p.pathname, any_dark)) == ['test-none.sh', 'none-none.sh']
